from __future__ import absolute_import, division, print_function, unicode_literals

from . import decorator as logdecorator
from .logger import debug, DEBUG, error, ERROR, get_logger, info, INFO, init, warn, WARNING
